import json
from random import randrange
from pprint import pprint
from typing import List
from enum import Enum, auto
import requests
from loguru import logger
from requests.auth import HTTPBasicAuth, HTTPDigestAuth
import xmltodict
from ocpy.api.api_client import OpenCastBaseApiClient


class Service:
    def __init__(self, data):
        self.data = data

    def __str__(self):
        return json.dumps(self.data, sort_keys=True, indent=4, separators=(",", ": "))

    def __repr__(self):
        return self.__str__()

    def get_url(self):
        return self.get_host() + self.get_path()

    def is_active(self) -> bool:
        return self.data["active"]

    def is_jobproducer(self) -> bool:
        return self.data["jobproducer"]

    def is_maintenance(self) -> bool:
        return self.data["maintenance"]

    def is_online(self) -> bool:
        return self.data["online"]

    def get_error_state_trigger(self) -> str:
        return self.data["error_state_trigger"]

    def get_host(self) -> str:
        return self.data["host"]

    def get_onlinefrom(self) -> str:
        return self.data["onlinefrom"]

    def get_path(self) -> str:
        return self.data["path"]

    def get_service_state(self) -> str:
        return self.data["service_state"]

    def get_state_changed(self) -> str:
        return self.data["state_changed"]

    def get_type(self) -> str:
        return self.data["type"]

    def get_warning_state_trigger(self) -> int:
        return self.data["warning_state_trigger"]


class Host:
    def __init__(self, data):
        self.data = data

    def __str__(self):
        return json.dumps(self.data, sort_keys=True, indent=4, separators=(",", ": "))

    def __repr__(self):
        return self.__str__()

    def is_active(self) -> bool:
        return self.data["active"]

    def is_in_maintenance(self) -> bool:
        return self.data["maintenance"]

    def is_online(self) -> bool:
        return self.data["online"]

    def get_address(self) -> str:
        return self.data["address"]

    def get_base_url(self) -> str:
        return self.data["base_url"]

    def get_cores(self) -> int:
        return self.data["core"]

    def get_max_load(self) -> int:
        return self.data["max_load"]

    def get_memory(self) -> int:
        return self.data["memory"]

    def get_memory_mb(self) -> int:
        return int(self.get_memory() / (1024 * 1024))


class ServiceType(Enum):
    adminui_endpoint_AclEndpoint = auto()
    adminui_endpoint_GroupsEndpoint = auto()
    adminui_endpoint_JobEndpoint = auto()
    adminui_endpoint_ListProvidersEndpoint = auto()
    adminui_endpoint_PresetsEndpoint = auto()
    adminui_endpoint_SeriesEndpoint = auto()
    adminui_endpoint_ServerEndpoint = auto()
    adminui_endpoint_ServicesEndpoint = auto()
    adminui_endpoint_TasksEndpoint = auto()
    adminui_endpoint_ThemesEndpoint = auto()
    adminui_endpoint_UserSettingsEndpoint = auto()
    adminui_endpoint_UsersEndpoint = auto()
    adminui_endpoint_event = auto()
    adminui_endpoint_index = auto()
    adminui_endpoint_tools = auto()
    adminui_i18n = auto()
    annotation = auto()
    assetmanager = auto()
    authorization_xacml_manager = auto()
    caption = auto()
    capture_admin = auto()
    composer = auto()
    coverimage = auto()
    distribution_acl = auto()
    distribution_aws_s3 = auto()
    distribution_download = auto()
    distribution_streaming = auto()
    engage_plugin_manager = auto()
    execute = auto()
    external = auto()
    external_events = auto()
    external_groups = auto()
    external_security = auto()
    feed_impl_FeedServiceImpl = auto()
    fles = auto()
    fileupload = auto()
    groups = auto()
    incident = auto()
    info = auto()
    ingest = auto()
    inspection = auto()
    kernel_bundleinfo = auto()
    message_broker_endpoint = auto()
    nop = auto()
    oaipmhinfo = auto()
    organization = auto()
    publication_oaipmh = auto()
    publication_youtube = auto()
    scheduler = auto()
    search = auto()
    series = auto()
    serviceregistry = auto()
    silencedetection = auto()
    smil = auto()
    sox = auto()
    staticfiles = auto()
    textanalyzer = auto()
    timelinepreviews = auto()
    transcription_ibmwatson = auto()
    userdirectory_endpoint_UserEndpoint = auto()
    userdirectory_roles = auto()
    userdirectory_users = auto()
    usertracking = auto()
    videoeditor = auto()
    videosegmenter = auto()
    waveform = auto()
    workflow = auto()

    def to_name(self, prefix="org.opencastproject."):
        return prefix + self.name.replace("_", ".")


class ServiceApi(OpenCastBaseApiClient):
    def __init__(self, user=None, password=None, server_url=None, **_kwargs):
        super().__init__(user, password, server_url)
        self.base_url = self.server_url + "/services"

    def get_active_jobs(self, **kwargs) -> dict:
        url = self.base_url + "/activeJobs.json"
        res = requests.get(
            url,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            return res.json()
        if res is not None:
            raise Exception(str(res))
        raise Exception("Could not get active jobs!")

        # TODO job children (in job)
        # GET /job/{id}.json
        # and GET /job/{id}/children.json

    def get_available(self, service_type: ServiceType, **kwargs) -> List[Service]:
        services_res = []
        if isinstance(service_type, ServiceType):
            service_type = service_type.to_name()
        else:
            if "org.opencastproject." not in service_type:
                service_type = "org.opencastproject." + service_type
        url = self.base_url + "/available.json"
        res = requests.get(
            url,
            auth=HTTPBasicAuth(self.user, self.password),
            params={"serviceType": service_type},
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            services = res.json()["services"]
            pprint(services)
            if isinstance(services["service"], list):
                for s in services["service"]:
                    services_res.append(Service(s))
            else:
                services_res.append(Service(services["service"]))
            return services_res
        raise Exception("Could not get available services!")

    def get_count(self, **kwargs) -> int:
        url = self.base_url + "/count"
        res = requests.get(
            url,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            return res.json()
        raise Exception("Could not service count!")

    def get_current_load(self, **kwargs) -> int:
        url = self.base_url + "/currentload"
        res = requests.get(
            url,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            return xmltodict.parse(res.content)
        raise Exception("Could not current load!")

    def get_health(self, **kwargs) -> int:
        url = self.base_url + "/health.json"
        res = requests.get(
            url,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            return res.json()
        raise Exception("Could not get health!")

    def get_hosts(self, **kwargs) -> List[Host]:
        res_hosts = []
        url = self.base_url + "/hosts.json"
        res = requests.get(
            url,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            hosts = res.json()["hosts"]
            if isinstance(hosts["host"], dict):
                res_hosts.append(Host(hosts["host"]))
                return res_hosts
            for h in hosts["host"]:
                res_hosts.append(Host(h))
            return res_hosts
        raise Exception("Could not get health!")

    def get_max_concurrent_jobs(self, **kwargs) -> int:
        url = self.base_url + "/maxconcurrentjobs"
        res = requests.get(
            url,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            return res.content
        raise Exception("Could not get max concurrent jobs!")

    def get_max_load(self, **kwargs) -> int:
        url = self.base_url + "/maxload"
        res = requests.get(
            url,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            return res.content
        raise Exception("Could not get max load!")

    def get_own_load(self, **kwargs) -> float:
        url = self.base_url + "/ownload"
        res = requests.get(
            url,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            return float(res.content)
        raise Exception("Could not get own load!")

    def get_services(
        self, service_type: ServiceType = None, host: str = None, **kwargs
    ):
        if "service_type" in kwargs and service_type is None:
            service_type = kwargs.get("service_type")
        if "host" in kwargs and host is None:
            host = kwargs.get("host")
        if isinstance(service_type, ServiceType):
            service_type = service_type.to_name()
        else:
            if service_type is not None and "org.opencastproject." not in service_type:
                service_type = "org.opencastproject." + service_type
        params = {}
        if service_type is not None:
            params["serviceType"] = service_type
        if host is not None:
            params["host"] = host
        url = self.base_url + "/services.json"
        res = requests.get(
            url, auth=HTTPBasicAuth(self.user, self.password), params=params, **kwargs
        )
        if res.ok:
            return res.json()
        raise Exception("Could not get services!")

    def get_service_warnings(self, **kwargs) -> int:
        url = self.base_url + "/servicewarnings"
        res = requests.get(
            url,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            return int(res.content)
        raise Exception("Could not get service warnings!")

    def get_statistics(self, **kwargs) -> str:
        url = self.base_url + "/statistics.json"
        res = requests.get(
            url,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            return res.json()
        raise Exception("Could not get statistics!")

    def create_job(self):
        pass

    def delete_job(self):
        pass


def main():
    api = ServiceApi()
    api = ServiceApi(server_url="http://opencast-dev.bibliothek.kit.edu:8080")
    api = ServiceApi(
        server_url="http://localhost:8080", user="admin", password="opencast"
    )
    # pprint(api.get_active_jobs())
    # services = api.get_available("ingest")

    services = api.get_available(ServiceType.workflow)
    print(services)
    api = services.pop().get_url()

    print(api)
    exit()

    # services = api.get_available(ServiceType.distribution_aws_s3)
    # services = api.get_available(ServiceType.kernel_bundleinfo)
    # pprint(services)
    for s in services:
        print(s.get_url())
    service = services[randrange(0, len(services))]
    logger.info("Selecting ingest service to use: " + str(service))

    url = service.get_url() + "/createMediaPackage"
    print(url)
    # res = requests.get(url, auth=HTTPDigestAuth("admin", "opencast"), headers={'X-Requested-Auth': 'Digest'})
    # res = requests.get(url)
    res = requests.get(
        url,
        auth=HTTPDigestAuth("opencast_system_account", "CHANGE_ME"),
        headers={"X-Requested-Auth": "Digest"},
    )
    if res.ok:
        print(res.content)
        print(res)
    else:
        print(res)

    exit()
    # pprint(api.get_services(host='http://opencast-dev.bibliothek.kit.edu:8080'))

    print(api.get_current_load())
    pprint(api.get_health())
    hosts = api.get_hosts()
    print("hosts: " + str(hosts))
    for h in hosts:
        print(h.get_address())
        pprint(h.get_memory())

    pprint(api.get_max_concurrent_jobs())
    pprint(api.get_max_load())
    pprint(api.get_own_load())
    pprint(api.get_service_warnings())
    pprint(api.get_statistics())


if __name__ == "__main__":
    main()



import requests
from requests.auth import HTTPBasicAuth
import json
from pprint import pprint

from ocpy.api.api_client import OpenCastBaseApiClient


class User:
    def __init__(self, user, password, url, data):
        self.base_url = url
        self.user = user
        self.password = password
        self.data = data

    def __str__(self):
        return json.dumps(self.data, sort_keys=True, indent=4, separators=(",", ": "))

    def __repr__(self):
        return self.__str__()

    def get_email(self) -> str:
        return self.data["email"]

    def get_name(self) -> str:
        return self.data["name"]

    def get_provider(self) -> str:
        return self.data["provider"]

    def get_username(self) -> str:
        return self.data["username"]

    def get_user_role(self) -> str:
        return self.data["userRole"]


class InfoApi(OpenCastBaseApiClient):
    def __init__(self, user=None, password=None, server_url=None, **_kwargs):
        super().__init__(user, password, server_url)
        self.base_url = self.server_url + "/info"

    def get_api_info(self, **kwargs) -> dict:
        res = requests.get(
            self.base_url,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs,
        )
        if res.ok:
            return res.json()
        raise Exception("Could not get api info!")

    def get_components_info(self, **kwargs) -> dict:
        url = self.base_url + "/components.json"
        res = requests.get(
            url,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs,
        )
        if res.ok:
            return res.json()
        raise Exception("Could not get api version!")

    def get_user_info(self, **kwargs) -> User:
        url = self.base_url + "/me.json"
        res = requests.get(
            url,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs,
        )
        if res.ok:
            user = User(self.user, self.password, url, res.json())
            return user
        raise Exception("Could not get user info!")


def main():
    api = InfoApi()
    s = api.get_user_info()
    print(s)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""Opencast admin_ui ServiceAPI.

Base implementation of opencast admin_ui event api.
Please refer to und use external event API functions.
"""
import requests
from requests.auth import HTTPBasicAuth
import json
from pprint import pprint

from typing import List

from ocpy.api.api_client import OpenCastBaseApiClient


class Service:
    def __init__(self, data):
        self.data = data

    def __str__(self):
        return json.dumps(self.data, sort_keys=True, indent=4, separators=(",", ": "))

    def __repr__(self):
        return self.__str__()

    def get_completed(self) -> int:
        return self.data["completed"]

    def get_hostname(self) -> str:
        """

        Returns:

        """
        return self.data["hostname"]

    def get_mean_queue_time(self) -> int:
        """

        Returns:

        """
        return self.data["meanQueueTime"]

    def get_mean_run_time(self) -> int:
        return self.data["meanRunTime"]

    def get_queued(self) -> int:
        return self.data["queued"]

    def get_running(self) -> int:
        return self.data["running"]

    def get_name(self) -> str:
        return self.data["name"]

    def get_status(self) -> str:
        return self.data["status"]


class Services:
    def __init__(self, data):
        self.data = data

    def __str__(self):
        return json.dumps(self.data, sort_keys=True, indent=4, separators=(",", ": "))

    def __repr__(self):
        return self.__str__()

    def get_count(self) -> int:
        return self.data["count"]

    def get_limit(self) -> int:
        return self.data["limit"]

    def get_offset(self) -> int:
        return self.data["offset"]

    def get_total(self) -> int:
        return self.data["total"]

    def get_results(self) -> List[Service]:
        services = []
        for s in self.data["results"]:
            services.append(Service(s))
        return services


class AdminServiceApi(OpenCastBaseApiClient):
    def __init__(self, user=None, password=None, server_url=None):
        super().__init__(user, password, server_url)
        self.base_url = self.server_url + "/admin-ng/services"

    def get_services(self, **kwargs) -> dict:
        url = self.base_url + "/services.json"
        print(url)
        res = requests.get(url, auth=HTTPBasicAuth(self.user, self.password), **kwargs)
        print(res)
        if res.ok:
            return res.json()
        raise Exception("Could not get api version!")


def main():
    api = AdminServiceApi()
    pprint(api.get_services())


if __name__ == "__main__":
    main()



"""Tests for Admin EventAPI"""

from pprint import pformat
import unittest
import pytest

from loguru import logger
from ocpy.api.admin_ui.event import EventApi


@pytest.mark.usefixtures("setup_oc_connection")
class TestEventsClass(unittest.TestCase):
    """Test class for Admin EventAPI"""

    def test_get_services(self):
        """Test get services call"""
        admin_ev_api = EventApi()
        meta = admin_ev_api.get_new_metadata()
        self.assertIsNotNone(meta)
        self.assertIsInstance(meta, list)

    def test_get_assets(self):
        """Test get assets"""
        admin_ev_api = EventApi()
        assets = admin_ev_api.get_assets("7ea01428-7be2-4a4f-b961-43a225f20dfd")
        logger.info(pformat(assets))
        self.assertIsNotNone(assets)
        self.assertIsInstance(assets, dict)

    def test_get_attachments(self):
        """Test get attachments"""
        admin_ev_api = EventApi()
        attachments = admin_ev_api.get_attachments(
            "7ea01428-7be2-4a4f-b961-43a225f20dfd"
        )
        logger.info(pformat(attachments))
        self.assertIsNotNone(attachments)
        self.assertIsInstance(attachments, list)

    def test_get_mime_type(self):
        """Test get mime type"""
        admin_ev_api = EventApi()
        mime = admin_ev_api.get_mime_type("tests/data/test.vtt")
        logger.info(mime)

    def test_post_attachment(self):
        """Test post attachment"""
        admin_ev_api = EventApi()
        wf_id = admin_ev_api.post_attachment(
            "8209008d-c6e1-4ff6-96e2-98228b2a2e1c",
            "track_subtitles_option_de",
            "tests/data/test.vtt",
        )
        logger.info(pformat(wf_id))
        self.assertIsNotNone(wf_id)

    def test_post_subtitiles(self):
        """Test post subtitiles"""
        admin_ev_api = EventApi()
        wf_id = admin_ev_api.post_subtitles(
            "8c421523-e769-40df-839d-a778b5b7f4bc", "tests/data/test2.vtt"
        )
        logger.info(pformat(wf_id))
        self.assertIsNotNone(wf_id)



#  Copyright (c) 2019. Tobias Kurze

import json
from datetime import datetime
from typing import List, Optional

import pendulum


class Scheduling:
    def __init__(self, agent_id: str, start: str, end: str, inputs: Optional[List[str]] = None):
        self.agent_id = agent_id
        if isinstance(start, str):
            start = pendulum.parse(start)
        self.start = start
        if isinstance(end, str):
            end = pendulum.parse(end)
        self.end = end
        self.inputs = ["defaults"] if inputs is None else inputs

    def __str__(self):
        return json.dumps(self.get_scheduling_dict())

    def __repr__(self):
        return self.__str__()

    def get_scheduling_dict(self):
        return {
            "agent_id": self.agent_id,
            "inputs": self.inputs,
            "start": self.start.isoformat(),
            "end": self.end.isoformat()
        }

    def get_agent_id(self) -> str:
        return self.agent_id

    def get_inputs(self) -> List[str]:
        return self.inputs

    def get_start(self) -> datetime:
        return self.start

    def get_end(self) -> datetime:
        return self.end


if __name__ == '__main__':
    dt = pendulum.parse("2022-06-22T11:55:00Z")
    print(dt)
    print(type(dt))
    print(dt.timezone)
    print(dt.isoformat())

    s = Scheduling(agent_id="test SMP", start="2022-06-22T11:55:00Z", end="2022-06-22T12:59:00Z")
    print(s)



#  Copyright (c) 2023. Tobias Kurze
"""Search API tests."""
import pytest
import tests

from loguru import logger


@pytest.mark.usefixtures("setup_oc_connection")
class TestSearchApiClass:
    """Test the search API."""

    def test_search_api_episodes(self):
        """Get episodes from search API."""
        episodes = tests.SEARCH_API.get_episode(limit=1)
        logger.info(episodes)
        assert isinstance(episodes, dict)
        # test_acl = []

    @pytest.mark.skip(
        reason="deprecated call / format=xml not supported anymore or differently"
    )
    def test_search_api_episodes_xml(self):
        """Get episodes from search API in xml."""
        episodes = tests.SEARCH_API.get_episode(limit=1, format="xml")
        logger.info(episodes)
        assert isinstance(episodes, str)
        # test_acl = []

    def test_search_api_lucene(self):
        """Use lucene from search API."""
        episodes = tests.SEARCH_API.get_lucene(limit=1)
        logger.info(episodes)
        assert isinstance(episodes, dict)
        # test_acl = []

    def test_search_api_series(self):
        """Get series from search API."""
        episodes = tests.SEARCH_API.get_series(limit=1)
        logger.info(episodes)
        assert len(episodes) == 1
        # test_acl = []



from loguru import logger
from typing import Union

import requests
import datetime

from requests.auth import HTTPBasicAuth  # , HTTPDigestAuth

from ocpy.api.api_client import OpenCastBaseApiClient

epoch = datetime.datetime.fromtimestamp(0, tz=datetime.timezone.utc)


def unix_time_millis(dt):
    """Converts a datetime object to milliseconds since epoch."""
    return int((dt - epoch).total_seconds() * 1000.0)


class SigningApi(OpenCastBaseApiClient):
    def __init__(self, user=None, password=None, server_url=None, **_kwargs):
        super().__init__(user, password, server_url)
        self.base_url = self.server_url + "/signing"

    def accepts(self, url: str, **kwargs):
        u = self.base_url + "/accepts"
        logger.debug(f"url: {u}")
        res = requests.get(
            u,
            auth=HTTPBasicAuth(self.user, self.password),
            params={"baseUrl": url},
            timeout=kwargs.pop("timeout", 30),
        )
        if res.ok:
            return res.text.lower() == "true"
        logger.error(f"Error occured: {res.text} (status_code: {res.status_code})")
        return False

    def sign(self, url: str, valid_until: Union[None, int] = None, **kwargs):
        if valid_until is None:
            valid_until = (
                unix_time_millis(datetime.datetime.now(tz=datetime.timezone.utc))
                + 60 * 60 * 1000
            )
        logger.debug(f"valid until: {valid_until} (unix_time_millis)")
        u = self.base_url + "/sign"
        params = {
            "baseUrl": url,
            "validUntil": valid_until,
            "validFrom": 0,
            "ipAddr": None,
        }
        logger.debug(f"url: {u}")
        logger.debug(f"params: {params}")
        res = requests.get(
            u,
            auth=HTTPBasicAuth(self.user, self.password),
            params=params,
            timeout=kwargs.pop("timeout", 30),
        )
        if res.ok:
            return res.text
        logger.error(f"Error occured: {res.text} (status_code: {res.status_code})")
        return None



import requests
import xml
from requests.auth import HTTPBasicAuth
import json

from ocpy.api.api_client import OpenCastBaseApiClient
from ocpy.model.mediapackage import MediaPackage


class AssetsApi(OpenCastBaseApiClient):
    def __init__(self, user=None, password=None, server_url=None, **_kwargs):
        super().__init__(user, password, server_url)
        self.base_url = self.server_url + "/assets"

    def get_episode_mp(self, mp_id, **kwargs) -> MediaPackage:
        res = requests.get(
            self.base_url + "/episode/" + mp_id,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs
        )
        if res.ok:
            try:
                return MediaPackage(res.text)
            except xml.parsers.expat.ExpatError:
                raise Exception(
                    "No valid XML response (probably authentication failure)!"
                )
        raise Exception("Could not get job info!")
    
    def post_mp_xml(self, mp_xml, **kwargs) -> bool:
        data = {"mediapackage": mp_xml}
        res = requests.post(
            self.base_url + "/add",
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            data=data
        )
        if res.ok:
            return True
        raise Exception("Could not post XML!")



#  Copyright (c) 2019. Tobias Kurze

from functools import total_ordering
import json
from typing import List


class Action:
    def __init__(self, action_string):
        self.action_string = action_string.lower()

    def __str__(self):
        return json.dumps({"action": self.action_string})

    def __repr__(self):
        return self.__str__()

    def get_action_string(self):
        return self.action_string.lower()

    @staticmethod
    def get_read_action():
        return Action("read")

    @staticmethod
    def get_write_action():
        return Action("write")


@total_ordering
class ACL:
    def __init__(self, allow: bool, role: str, action: Action):
        self.allow = allow
        self.role = role
        if isinstance(action, str):
            action = Action(action)
        self.action = action

    def __str__(self):
        return json.dumps(self.get_acl_dict())

    def __repr__(self):
        return self.__str__()

    def get_acl_dict(self):
        return {
            "allow": self.allow,
            "role": self.role,
            "action": self.action.get_action_string(),
        }

    def __eq__(self, other) -> bool:
        return (
            self.role.lower(),
            self.action.get_action_string().lower(),
            self.allow,
        ) == (
            other.role.lower(),
            other.action.get_action_string().lower(),
            other.allow,
        )

    def __lt__(self, other) -> bool:
        return (self.role.lower(), self.action.get_action_string().lower()) < (
            other.role.lower(),
            other.action.get_action_string().lower(),
        )

    @staticmethod
    def get_read_acl(role: str, as_list=True):
        if not as_list:
            return ACL(True, role, Action.get_read_action())
        return [ACL(True, role, Action.get_read_action())]

    @staticmethod
    def get_write_acl(role: str, as_list=True):
        if not as_list:
            return ACL(True, role, Action.get_write_action())
        return [ACL(True, role, Action.get_write_action())]

    @staticmethod
    def get_read_write_acls(role: str) -> List["ACL"]:
        return [ACL.get_read_acl(role, False), ACL.get_write_acl(role, False)]

    def get_role(self):
        return self.role

    def is_allowed(self):
        return self.allow

    def get_action(self):
        return self.action



"""API Client for the (external) series endpoint of the Opencast API."""

from urllib.parse import urlparse
import json
from typing import Generator, List, Optional, Union
from datetime import datetime
import pendulum
import requests
from requests.auth import HTTPBasicAuth
from loguru import logger


from ocpy import OcPyRequestException
from ocpy.api.api_client import OpenCastBaseApiClient
from ocpy.model.acl import ACL, Action


class Series:
    """Class represnting a series in Opencast."""

    def __init__(self, user, password, url, data) -> None:
        self.base_url = url
        self.user = user
        self.password = password
        self.data = data

        self._augment_data()

    def __str__(self) -> str:
        return json.dumps(
            self.data,
            sort_keys=True,
            indent=4,
            separators=(",", ": "),
            default=str,
        )

    def __repr__(self) -> str:
        return self.__str__()

    def _augment_data(self) -> None:
        if "created" in self.data:
            self.data["created"] = pendulum.parse(self.data["created"])

    def get_acl(self, **kwargs) -> Optional[List[ACL]]:
        res = self.data.get("acl", None)
        if res is None:
            url = self.base_url + "/acl"
            res = requests.get(
                url,
                timeout=kwargs.pop("timeout", 30),
                auth=HTTPBasicAuth(self.user, self.password),
            )
            if not res.ok:
                return None
            res = res.json()
            self.data["acl"] = res
        acls = []
        for a in res:
            acls.append(ACL(a["allow"], a["role"], Action(a["action"])))
        return acls

    def get_metadata(self, raw=False, **kwargs):
        url = self.base_url + "/metadata"
        res = requests.get(
            url,
            timeout=kwargs.pop("timeout", 30),
            auth=HTTPBasicAuth(self.user, self.password),
        )
        if res.ok:
            if raw:
                return res.json()
            meta_result = dict()
            for r in res.json():
                # merge metadata
                meta_result = {
                    **meta_result,
                    **{m["id"]: m["value"] for m in r["fields"] if "value" in m},
                }
            return meta_result
        return None

    def set_acl(self, acl_list, override=False, **kwargs):
        url = self.base_url + "/acl"
        if isinstance(acl_list, List):
            acls = []
            for a in acl_list:
                if isinstance(a, ACL):
                    acls.append(a.get_acl_dict())
                else:
                    acls.append(a)
            acl_list = acls
        if isinstance(acl_list, ACL):
            acl_list = [acl_list.get_acl_dict()]

        data = {
            "acl": json.dumps(acl_list),
            "override": "true" if override else "false",
            # "override": True if override else False,
        }
        logger.debug(f"set acl data: {data}")
        res = requests.put(
            url,
            timeout=kwargs.pop("timeout", 30),
            data=data,
            auth=HTTPBasicAuth(self.user, self.password),
        )
        if res.ok:
            return res.json()
        raise OcPyRequestException(
            "could not set ACL! ("
            + res.text
            + " ["
            + str(res.status_code)
            + "])"
            + "acl: "
            + json.dumps(acl_list)
        )

    def add_to_acl(self, acl: Union[ACL, List[ACL]], override=False):
        logger.debug(f"adding to acl: {acl}")
        actual_acl = self.get_acl()
        if not isinstance(acl, List):
            acl = [acl]
        for n_a in acl:
            found = False
            for a_a in actual_acl:
                if a_a == n_a:
                    found = True
                    break
            if not found:
                actual_acl.append(n_a)

        return self.set_acl(actual_acl, override=override)

    def remove_from_acl(self, acl: ACL):
        actual_acl = self.get_acl()

        if not isinstance(acl, List):
            acl = [acl]
        for aa in actual_acl:
            for na in acl:
                if aa == na:
                    actual_acl.remove(na)
        return self.set_acl(actual_acl)

    def get_properties(self, **kwargs):
        url = self.base_url + "/properties"
        res = requests.get(
            url,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
        )
        if res.ok:
            return res.json()
        return None

    def get_contributors(self):
        return self.data["contributors"]

    def get_created(self):
        return self.data["created"]

    def get_creator(self):
        return self.data["creator"]

    def get_identifier(self):
        return self.data["identifier"]

    def get_organizers(self):
        return self.data["organizers"]

    def get_publishers(self):
        return self.data["publishers"]

    def get_subjects(self):
        return self.data["subjects"]

    def get_title(self):
        return self.data["title"]

    def get_events(self, limit=100, **kwargs):
        from ocpy.api import EventsApi  # pylint: disable=import-outside-toplevel

        parsed_url = urlparse(self.base_url)
        url = f"{parsed_url.scheme}://{parsed_url.netloc}"
        ev_api = EventsApi(self.user, self.password, url)
        return ev_api.get_events_part_of(self, limit, **kwargs)

    def delete(self):
        res = requests.delete(
            self.base_url,
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=30,
        )
        if res.ok:
            return "ok"
        raise OcPyRequestException("could not delete series!", response=res)


class SeriesApi(OpenCastBaseApiClient):
    """API Client for the (external) series endpoint of the Opencast API."""

    def __init__(self, user=None, password=None, server_url=None, **_kwargs):
        super().__init__(user, password, server_url)
        self.base_url = self.server_url + "/api/series"

    def get_series(
        self,
        limit=100,
        offset=0,
        with_acl=False,
        series_filter=None,
        only_with_write_access=False,
        **kwargs,
    ) -> List[Series]:
        """
        :param limit:
        :param offset:
        :param with_acl:
        :param series_filter:
        :param onlyWithWriteAccess:
        :param kwargs:
        :return: List[Series]
        """
        url = self.base_url.rstrip("/") + "/"
        logger.trace(f"getting series url: {url}")
        parameters = {
            "limit": limit,
            "offset": offset,
            "sort": "",
            "filter": "",
            "withacl": str(with_acl).lower(),
            "onlyWithWriteAccess": str(only_with_write_access).lower(),
        }
        parameters = {k: v for k, v in parameters.items() if v != ""}

        logger.trace(f"getting series parameters: {parameters}")
        if series_filter:
            if isinstance(series_filter, SeriesApi.Filter):
                parameters["filter"] = series_filter.get_filter_string()
            else:
                parameters["filter"] = series_filter
        results = []
        req = requests.Request(
            method="GET",
            url=url,
            auth=HTTPBasicAuth(self.user, self.password),
            params=parameters,
            **kwargs,
        )
        prepared = req.prepare()
        logger.trace(f"getting series prepared url: {prepared.url}")
        res = requests.Session().send(
            prepared,
            timeout=kwargs.pop("timeout", 30),
        )
        if res.ok:
            for e in res.json():
                results.append(
                    Series(
                        self.user,
                        self.password,
                        url + e["identifier"],
                        e,
                    )
                )
            return results
        raise OcPyRequestException("could not get series!", response=res)

    def get_all_series(
        self,
        with_acl=False,
        only_with_write_access=False,
        series_filter=None,
        batch_size=100,
        print_progress=False,
        generator=False,
    ) -> Union[List[Series], Generator[Series, None, None]]:
        series_generator = self.get_all_series_generator(
            with_acl=with_acl,
            only_with_write_access=only_with_write_access,
            series_filter=series_filter,
            batch_size=batch_size,
            print_progress=print_progress,
        )
        if generator:
            return series_generator
        return list(series_generator)

    def get_all_series_generator(
        self,
        with_acl=False,
        only_with_write_access=False,
        series_filter=None,
        batch_size=100,
        print_progress=False,
    ) -> Generator[Series, None, None]:
        count = 0
        while True:
            if print_progress:
                logger.info(f"requesting {batch_size} starting from offset {count}")
            res = self.get_series(
                with_acl=with_acl,
                only_with_write_access=only_with_write_access,
                series_filter=series_filter,
                limit=batch_size,
                offset=count,
            )

            if res is None or len(res) <= 0:
                logger.debug("DONE; got no more series")
                break
            count += len(res)
            for r in res:
                yield r
        return

    def get_series_acl(self, series_id, **kwargs) -> List[ACL]:
        res = requests.get(
            self.base_url + f"/{series_id}/acl",
            auth=HTTPBasicAuth(self.user, self.password),
            timeout=kwargs.pop("timeout", 30),
            **kwargs,
        )
        acls = []
        if res.ok:
            for a in res.json():
                acls.append(ACL(a["allow"], a["role"], Action(a["action"])))
            return acls
        raise OcPyRequestException("could not get series acls!", response=res)

    def get_series_by_id(
        self,
        series_id,
        with_acl=False,
        **kwargs,
    ) -> Series:
        parameters = {"withacl": with_acl}
        res = requests.get(
            self.base_url + "/" + series_id,
            auth=HTTPBasicAuth(self.user, self.password),
            params=parameters,
            timeout=kwargs.pop("timeout", 30),
            **kwargs,
        )
        if res.ok:
            series = res.json()
            return Series(
                self.user,
                self.password,
                self.base_url + "/" + series["identifier"],
                series,
            )
        logger.error(f"{res.status_code}: {res.text}")
        raise OcPyRequestException("could not get series!", response=res)

    def create_series(self, acl, metadata, theme=None, **kwargs) -> Series:
        if isinstance(acl, List):
            acls = []
            for a in acl:
                if isinstance(a, ACL):
                    acls.append(a.get_acl_dict())
                else:
                    acls.append(a)
            acl = acls
        if isinstance(acl, ACL):
            acl = acl.get_acl_dict()
        data = {
            "acl": json.dumps(acl),
            "metadata": json.dumps(metadata),
            "theme": theme,
        }

        res = requests.post(
            self.base_url + "/",
            timeout=kwargs.pop("timeout", 30),
            auth=HTTPBasicAuth(self.user, self.password),
            data=data,
            **kwargs,
        )
        if res.ok:
            series = res.json()
            return Series(
                self.user,
                self.password,
                self.base_url + "/" + series["identifier"],
                series,
            )
        raise OcPyRequestException(
            "could not create event! (" + res.text + ")", response=res
        )

    class Filter:
        """Filter class for series."""

        def __init__(self):
            self.filter_string = None

        def update_filter_string(self, update_string):
            if self.filter_string is None:
                self.filter_string = update_string
            else:
                self.filter_string += "," + update_string

        def set_contributors_filter(self, contributors):
            self.update_filter_string(f"contributors:{contributors}")
            return self

        def set_creator_filter(self, creator):
            self.update_filter_string(f"creator:{creator}")
            return self

        def set_creation_date_filter(self, date1: datetime, date2):
            d1 = str(date1).replace("+00:00", "Z")
            d2 = str(date2).replace("+00:00", "Z")
            self.update_filter_string(f"creationDate:{d1}/{d2}")
            return self

        def set_language_filter(self, language):
            self.update_filter_string(f"language:{language}")
            return self

        def set_license_filter(self, oc_license):
            self.update_filter_string(f"license:{oc_license}")
            return self

        def set_organizers_filter(self, organizers):
            self.update_filter_string(f"organizers:{organizers}")
            return self

        def set_managed_acl_filter(self, managed_acl):
            self.update_filter_string(f"managedAcl:{managed_acl}")
            return self

        def set_subject_filter(self, subject):
            self.update_filter_string(f"subject:{subject}")
            return self

        def set_text_filter_filter(self, text_filter):
            self.update_filter_string(f"textFilter:{text_filter}")
            return self

        def set_title_filter(self, title):
            self.update_filter_string(f"title:{title}")
            return self

        def get_filter_string(self):
            return self.filter_string



#  Copyright (c) 2019. Tobias Kurze
import logging
import os
import time
import types
import unittest
from typing import Union

import pytest

from ocpy import OcPyRequestException
from ocpy.api import get_events_api_client
from ocpy.api.external import events
from ocpy.api.external.events import Event, EventsApi
from ocpy.model.acl import ACL
import tests

_test_event: Union[None, Event] = None


@pytest.fixture(autouse=False, scope="class")
def create_test_event():
    global _test_event
    logging.info("XXX test_event fixture")
    test_meta = [
        {
            "fields": [
                {
                    "id": "title",
                    "label": "EVENTS.EVENTS.DETAILS.METADATA.TITLE",
                    "readOnly": False,
                    "required": True,
                    "type": "text",
                    "value": "Testevent",
                }
            ],
            "flavor": "dublincore/episode",
        }
    ]

    test_acl = ACL.get_read_write_acls("ROLE_TEST_USER")
    # test_acl = []

    processing_info = {
        "configuration": {
            "comment": "false",
            "publishLive": "false",
            "publishToMediaModule": "true",
            "publishToOaiPmh": "true",
        },
        "workflow": "fast",
    }
    # "workflow": "import"
    # "workflow": "fast"}

    test_video_file = os.path.abspath(
        os.path.join(tests.__file__, os.path.pardir, "assets", "test_video.m4v")
    )
    _test_event = tests.EV_API.create_event(
        test_acl, test_meta, processing_info, test_video_file
    )
    yield _test_event
    print("XXX deleting")
    _test_event.delete()


@pytest.mark.usefixtures("setup_oc_connection")
class TestEventsClass(unittest.TestCase):
    @pytest.mark.skip(reason="skipped while event creation is disabled")
    def test_acl_add(self):
        ev = _test_event
        self.assertIsNotNone(ev, "Event must not be None")
        num_acls = len(ev.get_acl())
        logging.info(num_acls)
        # new events can't be modified immediately (e.g. ACL), so wait...
        failed = False
        try_counter = 0
        while not failed:
            try:
                ev.add_to_acl(ACL.get_read_acl("ROLE_ANONYMOUS"))
            except OcPyRequestException as e:
                if e.get_code() == 403:
                    logging.warning(
                        "event may be processing... can't modify ACL -> have to try again...(sleeping 30s)"
                    )
                    time.sleep(30)
            try_counter += 1
            if try_counter > 10:
                failed = True

        assert len(ev.get_acl()) > num_acls

    def test_get_one_event(self):
        ev = tests.EV_API.get_events(limit=1, offset=0)
        self.assertGreaterEqual(len(ev), 1)

    def test_event_filter(self):
        evs = tests.EV_API.get_events(
            events_filter=EventsApi.Filter().set_text_filter_filter("bunny")
        )
        self.assertGreaterEqual(len(evs), 1)

    def test_get_all_events_non_generator(self):
        evs = tests.EV_API.get_all_events(generator=False)
        self.assertGreaterEqual(len(evs), 1)

    def test_get_all_events_generator(self):
        evs = tests.EV_API.get_all_events(batch_size=2, generator=True)
        self.assertIsInstance(evs, types.GeneratorType)
        ev = next(evs)
        self.assertIsInstance(ev, Event)

    def test_add_caption_track(self):
        ev: Event = tests.EV_API.get_event("f4095f6c-5182-4647-a3ce-3d183b00dedb")

        self.assertIsNotNone(ev, "Event must not be None")
        self.assertIsInstance(ev, Event, "Event must be an instance of Event")
        logging.info(f"Event: {ev.get_identifier()}")
        caption_file = os.path.abspath(
            os.path.join(tests.__file__, os.path.pardir, "assets", "test.vtt")
        )
        ok = tests.EV_API.add_track(
            event_id=ev.get_identifier(),
            overwrite_existing=True,
            flavor="captions/test",
            track_file=caption_file,
            tags=[
                "archive",
                "subtitles",
                "lang:de",
                "type:subtitle",
                "generator-type:auto",
            ],
        )
        self.assertTrue(ok, "Adding caption track failed")
        logging.info("all good")


"""
def setup_func():
    global test_event
    print("setup func ...")
    test_event = create_test_event()


def teardown_func():
    print("teardown func ...")
    global test_event
    print(test_event.delete())
"""

# @with_setup(setup_func, teardown_func)
# def tests():
#    ev_api.get_all_events()
